@@ -44,10 +44,13 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+/* This is for bonobo_dock_item_set_locked */
+#define BONOBO_UI_INTERNAL
+
 /* Must be before all other gnome includes!! */
 #include "gnome-i18nP.h"
 
-
+#include <gconf/gconf-client.h>
 #include <libgnome/gnome-program.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-config.h>
@@ -208,10 +211,7 @@ gnome_app_instance_init (GnomeApp *app)
 	gtk_box_pack_start (GTK_BOX (app->vbox), app->dock,
 			    TRUE, TRUE, 0);
 
-	gtk_signal_connect (GTK_OBJECT (app->dock),
-			    "layout_changed",
-			    GTK_SIGNAL_FUNC (layout_changed),
-			    (gpointer) app);
+	g_signal_connect (app->dock, "layout_changed", G_CALLBACK (layout_changed), app);
 	
 	app->enable_layout_config = TRUE;
 
@@ -227,6 +227,62 @@ gnome_app_instance_init (GnomeApp *app)
 	}
 }
 
+static void
+remove_notification_cb (BonoboDockItem *item, gpointer data)
+{
+	GConfClient *client;
+	guint notify_id;
+
+	notify_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "gnome-app-gconf-notify-id"));
+
+	if (notify_id == 0)
+		return;
+	
+	client = gconf_client_get_default ();
+	gconf_client_notify_remove (client, notify_id);
+	g_object_unref (client);
+	g_object_set_data (G_OBJECT (item), "gnome-app-gconf-notify-id", NULL);
+	
+}
+
+static void
+dock_item_changed_notify (GConfClient* client,
+			  guint cnxn_id,
+			  GConfEntry *entry,
+			  gpointer user_data)
+{
+	BonoboDockItem *item;
+	guint notify_id;
+	
+	item = BONOBO_DOCK_ITEM (user_data);
+	notify_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "gnome-app-gconf-notify-id"));
+
+	if (notify_id == cnxn_id &&
+	    entry->value->type == GCONF_VALUE_BOOL) {
+		gboolean detachable;
+
+		detachable = gconf_value_get_bool (entry->value);
+		
+		/* Update */
+		bonobo_dock_item_set_locked (item, !detachable);
+	}
+}
+
+static void
+setup_notification_for_dock_item (BonoboDockItem *item, const char *key_name)
+{
+	guint notify_id;
+	GConfClient *client;
+
+	client = gconf_client_get_default ();
+	notify_id = gconf_client_notify_add (client, key_name,
+					     dock_item_changed_notify,
+					     item, NULL, NULL);
+
+	g_object_set_data (G_OBJECT (item), "gnome-app-gconf-notify-id", GINT_TO_POINTER (notify_id));
+	g_signal_connect (item, "destroy", G_CALLBACK (remove_notification_cb), NULL);
+}
+
 static void
 gnome_app_show (GtkWidget *widget)
 {
@@ -359,7 +415,7 @@ gnome_app_destroy (GtkObject *object)
 	}
 
 	if (app->layout != NULL) {
-		gtk_object_unref (GTK_OBJECT (app->layout));
+		g_object_unref (app->layout);
 		app->layout = NULL;
 	}
 
@@ -400,9 +456,7 @@ contents_parent_set (GtkWidget *widget, GtkWidget *previous_parent, gpointer dat
 
 	g_assert (previous_parent == app->dock);
 
-	gtk_signal_disconnect_by_func (GTK_OBJECT (widget),
-				       GTK_SIGNAL_FUNC (contents_parent_set),
-				       data);
+	g_signal_handlers_disconnect_by_func (widget, contents_parent_set, data);
 
 	app->contents = NULL;
 }
@@ -433,9 +487,9 @@ gnome_app_set_contents (GnomeApp *app, GtkWidget *contents)
 
 	if (new_contents == contents && new_contents != app->contents) {
 		gtk_widget_show (new_contents);
-		gtk_signal_connect (GTK_OBJECT (new_contents), "parent_set",
-				    GTK_SIGNAL_FUNC (contents_parent_set),
-				    app);
+		g_signal_connect (G_OBJECT (new_contents), "parent_set",
+				  G_CALLBACK (contents_parent_set),
+				  app);
 
 		app->contents = new_contents;
 	}
@@ -471,6 +525,7 @@ gnome_app_set_menus (GnomeApp *app, GtkMenuBar *menubar)
 
 	dock_item = bonobo_dock_item_new (GNOME_APP_MENUBAR_NAME,
 					 behavior);
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (dock_item), "/desktop/gnome/interface/menubar_detachable");
 	gtk_container_add (GTK_CONTAINER (dock_item), GTK_WIDGET (menubar));
 
 	app->menubar = GTK_WIDGET (menubar);
@@ -492,7 +547,7 @@ gnome_app_set_menus (GnomeApp *app, GtkMenuBar *menubar)
 	gtk_widget_show (GTK_WIDGET (menubar));
 	gtk_widget_show (GTK_WIDGET (dock_item));
 
-	ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+	ag = g_object_get_data(G_OBJECT(app), "GtkAccelGroup");
 	if (ag && !g_slist_find(gtk_accel_groups_from_object (G_OBJECT (app)), ag))
 	        gtk_window_add_accel_group(GTK_WINDOW(app), ag);
 }
@@ -594,7 +649,8 @@ gnome_app_add_toolbar (GnomeApp *app,
 	g_return_if_fail(toolbar != NULL);
 
 	dock_item = bonobo_dock_item_new (name, behavior);
-
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (dock_item), "/desktop/gnome/interface/toolbar_detachable");
+	
 	gtk_container_set_border_width (GTK_CONTAINER (toolbar), 1);
 	gtk_container_add (GTK_CONTAINER (dock_item), GTK_WIDGET (toolbar));
 
@@ -620,7 +676,7 @@ gnome_app_add_toolbar (GnomeApp *app,
 	gtk_widget_show (GTK_WIDGET (toolbar));
 	gtk_widget_show (GTK_WIDGET (dock_item));
 
-	ag = gtk_object_get_data(GTK_OBJECT(app), "GtkAccelGroup");
+	ag = g_object_get_data(G_OBJECT(app), "GtkAccelGroup");
 	if (ag && !g_slist_find(gtk_accel_groups_from_object (G_OBJECT (app)), ag))
 	        gtk_window_add_accel_group(GTK_WINDOW(app), ag);
 }
@@ -694,9 +750,7 @@ gnome_app_add_dock_item (GnomeApp *app,
 				     offset,
 				     FALSE);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (app->dock),
-				 "layout_changed",
-				 (gpointer) app);
+	g_signal_emit_by_name (app->dock, "layout_changed", app);
 }
 
 /**
@@ -731,6 +785,7 @@ gnome_app_add_docked (GnomeApp *app,
 	GtkWidget *item;
 
 	item = bonobo_dock_item_new (name, behavior);
+	setup_notification_for_dock_item (BONOBO_DOCK_ITEM (item), "/desktop/gnome/interface/toolbar_detachable");
 	gtk_container_add (GTK_CONTAINER (item), widget);
 	gnome_app_add_dock_item (app, BONOBO_DOCK_ITEM (item),
 				 placement, band_num, band_position, offset);
